package SAE.ApiREST.WebService.controller;

import SAE.ApiREST.WebService.exception.ArticleException;
import SAE.ApiREST.WebService.exception.CollectException;
import SAE.ApiREST.WebService.model.Article;
import SAE.ApiREST.WebService.model.Collect;
import SAE.ApiREST.WebService.service.ICollectionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Controller
@RequestMapping("/CollectWebService")
public class CollectController {
    @Autowired
    ICollectionService collectionService;

    public CollectController() {

    }

    // region Collection

    // region GET
    @GetMapping(value = "/getAllCollection", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody List<Collect> getAllCollection(){
        List<Collect> results = collectionService.getAllCollections();
        if(results.isEmpty()) {
            throw new CollectException("No collections available");
        }
        return results;
    }
    @GetMapping(value = "/getCollectionById/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<Collect> getCollectionById(@PathVariable(value = "id") Integer id){
        Collect results = collectionService.getCollectionById(id);
        if(results == null) {
            throw new CollectException("No collections available");
        }
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(id)).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(id)
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(id)
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    @GetMapping(value = "/getAllCollectionsByName/{name}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<List<Collect>> getAllCollectionsByName(@PathVariable(value = "name") String name){
        List<Collect> results = collectionService.getAllCollectionsByName(name);
        if(results.isEmpty()) {
            throw new CollectException("No collections available");
        }
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getAllCollectionsByName(name)).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    // endregion

    // region DELETE
    @DeleteMapping(value = "/deleteColletionById/{id}")
    public @ResponseBody EntityModel<Link> deleteColletionById(@RequestParam("id") Integer id){
        if(!collectionService.deleteColletionById(id)) {
            throw new CollectException("No collections available");
        }
        return EntityModel.of(linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @DeleteMapping(value = "/deleteColletionByName/{name}")
    public @ResponseBody EntityModel<Link> deleteColletionByName(@RequestParam("name") String name){
        if(!collectionService.deleteColletionByName(name)) {
            throw new CollectException("No collections available");
        }
        return EntityModel.of(linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @DeleteMapping(value = "/deleteAllColletionByName/{name}")
    public @ResponseBody EntityModel<Link> deleteAllColletionByName(@RequestParam("name") String name){
        if(!collectionService.deleteAllColletionByName(name)) {
            throw new CollectException("No collections available");
        }
        return EntityModel.of(linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    // endregion

    // region PUT
    @PutMapping(value = "/addCollection")
    public @ResponseBody EntityModel<Collect> addCollection(@RequestParam("collection") Collect collection){
        Collect results = collectionService.addCollection(collection);
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(results.getId())).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(results.getId())
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(results.getId())
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @PutMapping(value = "/addCollections")
    public @ResponseBody EntityModel<List<Collect>> addCollections(@RequestParam("collections") List<Collect> collections){
        List<Collect> results = collectionService.addCollections(collections);
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    // endregion

    // region POST
    @PostMapping(value="/modifyCollectionName", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<Collect> modifyCollectionName(@RequestParam("collection") Collect collection, @RequestParam("name") String name){
        Collect results = collectionService.modifyCollectionName(collection,name);
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(results.getId())).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(results.getId())
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(results.getId())
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @PostMapping(value="/modifyCollectionNameById/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<Collect> modifyCollectionNameById(@RequestParam("id") Integer id, @RequestParam("name") String name){
        Collect results = collectionService.modifyCollectionNameById(id,name);
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(id)).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(id)
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(id)
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    // endregion
    // endregion

    // region Article
    @GetMapping(value = "/getAllArticlesById/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<List<Article>> getAllArticlesById(@PathVariable(value = "id") Integer id){
        List<Article> results = collectionService.getAllArticlesById(id);
        if(results == null) {
            throw new ArticleException("No articles available");
        }

        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @PutMapping(value = "/addArticle", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<Collect> addArticle(@RequestParam("collection") Collect collection, @RequestParam("article") Article article){
        Collect results = collectionService.addArticle(collection,article);
        Article art = results.getAllArticles().get(results.getAllArticles().indexOf(article));

        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(results.getId())).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getTeachById(results.getTeacher().getId())).withSelfRel(),
                linkTo(methodOn(ArticleControler.class).getArticleById(art.getId())).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(results.getId())
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(results.getId())
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }

    @DeleteMapping(value = "/deleteArticle", produces = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody EntityModel<Collect> deleteArticle(@RequestParam("collection") Collect collection, @RequestParam("article") Article article){
        Collect results = collectionService.deleteArticle(collection,article);
        return EntityModel.of(results,
                linkTo(methodOn(CollectController.class).getCollectionById(results.getId())).withSelfRel(),
                linkTo(methodOn(CollectController.class).getAllArticlesById(results.getId())
                ).withRel("getAllArticlesById"),
                linkTo(methodOn(CollectController.class).deleteColletionById(results.getId())
                ).withRel("deleteColletionById"),
                linkTo(methodOn(CollectController.class).deleteAllColletionByName(results.getName())
                ).withRel("deleteAllColletionByName"),
                linkTo(methodOn(CollectController.class).getAllCollection()
                ).withRel("getAllCollection"));
    }
    //  endregion
}
